# Copyright (c) 2001-2003 Alexander Kanavin. All rights reserved.

"""
This module contains configuration classes for pyslsk.
"""

import ConfigParser
import string
import os.path
import cPickle

class Config:
    """ 
    This class holds configuration information and provides the 
    following methods:

    needConfig() - returns true if configuration information is incomplete
    readConfig() - reads configuration information from ~/.pyslsk
    setConfig(config_info_dict) - sets configuration information
    writeConfig - writes configuration information to ~/.pyslsk

    The actual configuration information is stored as a two-level dictionary.
    First-level keys are config sections, second-level keys are config 
    parameters.
    """
    def __init__(self, filename):
        self.filename = filename
	self.parser = ConfigParser.ConfigParser()
	self.parser.read([self.filename])
        self.sections = {"server":{"server":('mail.slsk.org', 2240), \
	"login":None,"passw":None, \
	"autosearch":[], \
	"portrange": (2234,2239), "enc":"utf-8","userlist":[], \
	"banlist":[], "ignorelist":[],"autojoin":["pyslsk"],"autoaway":15}, \
	"transfers":{"downloaddir":None,"sharedownloaddir":1,"shared":None, \
	"uploadbandwidth":100,"uselimit":0,"uploadlimit":100,"limitby":1,
	"usecustomban":0,"customban":"don't bother to retry", \
	"downloads":[],"sharedfiles":{},"sharedfilesstreams":{}, \
	"sharedindex":{},"sharedmtimes":{},"rescanonstartup":0}, \
	"userinfo":{"descr":"''","pic":""},"logging": \
	{"logsdir":os.path.expanduser("~"),"privatechat":0,"chatrooms":0}, \
	"searches":{"maxresults":50}}
	try:
	    f = open(filename+".shares")
	    self.sharesdb = cPickle.load(f)
	    f.close()
	except:
	    self.sharesdb = None
	    
    def needConfig(self):
	for i in self.sections.keys():
	    for j in self.sections[i].keys():
		if self.sections[i][j] is None or self.sections[i][j] == '' and i != "userinfo":
		    return 1
	return 0

    def readConfig(self):
	for i in self.parser.sections():
	    for j in self.parser.options(i):
		val = self.parser.get(i,j, raw = 1)
		if i not in self.sections.keys():
		    print "Bogus config section:",i
		elif j not in self.sections[i].keys():
		    print "Bogus config option",j,"section",i
		elif j in ['login','passw','enc','downloaddir','customban','descr','pic','logsdir']:
		    self.sections[i][j] = val
		else:
		    try:
		        self.sections[i][j] = eval(val)
		    except:
			self.sections[i][j] = None
			raise
	if self.sharesdb is not None:
	    self.sections["transfers"]["sharedfiles"] = self.sharesdb[0]
	    self.sections["transfers"]["sharedfilesstreams"] = self.sharesdb[1]
	    self.sections["transfers"]["sharedindex"] = self.sharesdb[2]
	    if len(self.sharesdb) > 3:
	        self.sections["transfers"]["sharedmtimes"] = self.sharesdb[3]
	if self.sections["server"]["server"] == ('mail.slsk.org',2242):
	    self.sections["server"]["server"] = ('mail.slsk.org', 2240)
 
    def writeConfig(self):
        for i in self.sections.keys():
            if not self.parser.has_section(i):
                self.parser.add_section(i)
            for j in self.sections[i].keys():
                if j not in ["sharedfiles","sharedfilesstreams","sharedindex","sharedmtimes"]:
                    self.parser.set(i,j,self.sections[i][j])
                else:
                    self.parser.remove_option(i,j)

        oldumask = os.umask(0077)
	f = open(self.filename,"w")
	self.parser.write(f)
	f.close()
        os.umask(oldumask)
        # A paranoid precaution since .pyslsk contains the password
        try:
            os.chmod(self.filename, 0600)
        except:
            pass

    def writeShares(self):
	sharesdb = [self.sections["transfers"]["sharedfiles"],self.sections["transfers"]["sharedfilesstreams"],self.sections["transfers"]["sharedindex"],self.sections["transfers"]["sharedmtimes"]]
	f = open(self.filename + ".shares","w")
	cPickle.dump(sharesdb, f)
	f.close()

